/************************************************************************
 *
 * filter.h : $Revision: 1.8.6.2 $
 *
 * (c) Copyright 1996-2005 Analog Devices, Inc.  All rights reserved.
 *
 ************************************************************************/
#pragma once
#ifndef __NO_BUILTIN
#pragma system_header /* filter.h */
#endif

#ifndef  __FILTER_DEFINED
#define  __FILTER_DEFINED


#include <fract_typedef.h>
#include <complex.h>


#ifdef __cplusplus
 extern "C" {
#endif 

typedef struct
{
    fract16 *h;    /*  filter coefficients            */
    fract16 *d;    /*  start of delay line            */
    fract16 *p;    /*  read/write pointer             */
    int k;         /*  number of coefficients         */
    int l;         /*  interpolation/decimation index */
} _fir_fr16_state;


typedef struct
{
    fract16 *c;    /* coefficients                   */
    fract16 *d;    /* start of delay line            */
    int k;         /* number of bi-quad stages       */
} _iir_fr16_state;


typedef struct
{
    fract16 *c;    /* coefficients                  */
    fract16 *d;    /* start of delay line           */
    fract16 *p;    /* read/write pointer            */
    int k;         /* 2*number of biquad stages + 1 */
} _iirdf1_fr16_state;


typedef struct
{
    int k;
    complex_fract16 *h;
    complex_fract16 *d;
    complex_fract16 *p;
} _cfir_fr16_state;

/* Macros */

#define fir_init(state, coeffs, delay, ncoeffs, index) \
    (state).h = (coeffs);  \
    (state).d = (delay);   \
    (state).p = (delay);   \
    (state).k = (ncoeffs); \
    (state).l = (index)

#define iir_init(state, coeffs, delay, stages) \
    (state).c = (coeffs); \
    (state).d = (delay);  \
    (state).k = (stages)

#define iirdf1_init(state, coeffs, delay, stages) \
    (state).c = (coeffs); \
    (state).d = (delay);  \
    (state).p = (delay);  \
    (state).k = (2*stages+1)

#define cfir_init(state, coeffs, delay, ncoeffs) \
    (state).h = (coeffs); \
    (state).d = (delay);  \
    (state).p = (delay);  \
    (state).k = (ncoeffs)

#define fir_state_fr16    _fir_fr16_state
#define iir_state_fr16    _iir_fr16_state
#define iirdf1_state_fr16 _iirdf1_fr16_state
#define cfir_state_fr16   _cfir_fr16_state


/* * * *        fir      * * * *
 *
 *    finite impulse response filter
 * 
 */

                       _Pragma("linkage_name __fir_fr16")
        void fir_fr16 (const fract16 _x[], fract16 _y[], int _n, 
                       _fir_fr16_state *_s);




/* * * *        iir      * * * *
 *
 *    Direct Form II Infinite Impulse Response filter
 * 
 */

                       _Pragma("linkage_name __iir_fr16")
        void iir_fr16 (const fract16 _x[], fract16 _y[], int _n, 
                       _iir_fr16_state *_s);



/* * * *        iirdf1      * * * *
 *
 *    Direct Form I Infinite Impulse Response filter
 *
 */

                       _Pragma("linkage_name __iirdf1_fr16")
        void iirdf1_fr16 (const fract16 _x[], fract16 _y[],
                          int _n, _iirdf1_fr16_state *_s);


                       _Pragma("linkage_name __coeff_iirdf1_fr16")
        void coeff_iirdf1_fr16 (const float _acoeff[], const float _bcoeff[],
                                fract16 _coeff[], int _nstages);


/* * * *        cfir      * * * *
 *
 *    complex finite impulse response filter
 * 
 */

                       _Pragma("linkage_name __cfir_fr16")
        void cfir_fr16 (const complex_fract16 _x[], complex_fract16 _y[], 
                        int _n, _cfir_fr16_state *_s);




/* * * *        fir_decimation      * * * *
 *
 *    FIR decimation filter
 * 
 */

                       _Pragma("linkage_name __fir_decima_fr16")
        void fir_decima_fr16 (const fract16 _x[], fract16 _y[], 
                        int _n, _fir_fr16_state *_s);




/* * * *        fir_interp      * * * *
 *
 *    FIR interpolation filter
 * 
 */

                       _Pragma("linkage_name __fir_interp_fr16")
        void fir_interp_fr16 (const fract16 _x[], fract16 _y[], 
                        int _n, _fir_fr16_state *_s);




/* * * *        convolve      * * * *
 *
 *    Convolution 1D
 * 
 */

                       _Pragma("linkage_name __convolve_fr16")
        void convolve_fr16 (const fract16 _a[], int _n, const fract16 _b[], 
                        int m, fract16 _c[]);




/* * * *        conv2d      * * * *
 *
 *    2D convolution
 * 
 */

                       _Pragma("linkage_name __conv2d_fr16")
        void conv2d_fr16 (const fract16 _a[], int _na, int _ma, const fract16 _b[], 
                        int _nb, int _mb, fract16 _c[]);




/* * * *        conv2d3x3      * * * *
 *
 *    3D convolution
 * 
 */

                       _Pragma("linkage_name __conv2d3x3_fr16")
        void conv2d3x3_fr16 (const fract16 _a[], int _na, int _ma, 
                             const fract16 _b[], fract16 _c[]);



/* * * *        twidfft      * * * *
 *
 *    twiddle functions
 *       twidfft_fr16        3/4 n sized twiddle table (use in Radix2 and Radix4 FFTs)
 *                                                                         <cos/sin>
 *
 *       twidfftrad2_fr16    1/2 n sized twiddle table (use in Radix2 FFT) <cos/sin>
 *       twidfftrad4_fr16    3/4 n sized twiddle table (use in Radix4 FFT) <cos/sin>
 *       twidfft2d_fr16          n sized twiddle table (use in 2D-FFT)     <cos/sin> 
 *
 *       twidfftf_fr16       3/4 n sized twiddle table (use in Radix4 FFTF <cos/-sin>
 */

                       _Pragma("linkage_name __twidfft_fr16")
        void twidfft_fr16 (complex_fract16 _w[], int _n);

                       _Pragma("linkage_name __twidfftrad2_fr16")
        void twidfftrad2_fr16 (complex_fract16 _w[], int _n);

                       _Pragma("linkage_name __twidfftrad4_fr16")
        void twidfftrad4_fr16 (complex_fract16 _w[], int _n);

                       _Pragma("linkage_name __twidfft2d_fr16")
        void twidfft2d_fr16 (complex_fract16 _w[], int _n);


                       _Pragma("linkage_name __twidfftf_fr16")
        void twidfftf_fr16 (complex_fract16 _w[], int _n);



/* * * *        cfftf_fr16      * * * *
 *
 *    Fast N point radix 4 complex input FFT
 *
 */
                       _Pragma("linkage_name __cfftf_fr16")
        void cfftf_fr16(const complex_fract16 *_in, complex_fract16 *_out,
                        const complex_fract16 _w[], int _wst, int _n);



/* * * *        cfftN      * * * *
 *
 *    N point radix 2 complex input FFT
 * 
 */

                       _Pragma("linkage_name __cfftN_fr16")
        void cfft_fr16 (const complex_fract16 *_in, complex_fract16 * _t, 
                              complex_fract16 *_out, const complex_fract16 _w[], 
                        int _wst, int _n, int _b, int _s);




/* * * *        rfftN      * * * *
 *
 *    N point radix 2 real input FFT
 * 
 */

                       _Pragma("linkage_name __rfftN_fr16")
        void rfft_fr16 (const fract16 *_in, complex_fract16 * _t, 
                        complex_fract16 *_out, const complex_fract16 _w[], 
                        int _wst, int _n, int _b, int _s);




/* * * *        ifftN      * * * *
 *
 *    N point radix 2 inverse FFT
 * 
 */

                       _Pragma("linkage_name __ifftN_fr16")
        void ifft_fr16 (const complex_fract16 *_in, complex_fract16 * _t, 
                        complex_fract16 *_out, const complex_fract16 _w[], 
                        int _wst, int _n, int _b, int _s );




/* * * *        cfftrad4      * * * *
 *
 *    N point radix 4 complex input FFT
 * 
 */

                       _Pragma("linkage_name __cfftrad4_fr16")
        void cfftrad4_fr16 (const complex_fract16 *_in, complex_fract16 * _t, 
                        complex_fract16 *_out, const complex_fract16 _w[], 
                        int _wst, int _n, int _b, int _s);




/* * * *        rfftrad4      * * * *
 *
 *    N point radix 4 real input FFT
 * 
 */

                       _Pragma("linkage_name __rfftrad4_fr16")
        void rfftrad4_fr16 (const fract16 *_in, complex_fract16 * _t, 
                        complex_fract16 *_out, const complex_fract16 _w[], 
                        int _wst, int _n, int _b, int _s);




/* * * *        ifftrad4      * * * *
 *
 *    N point radix 4 inverse FFT
 * 
 */

                       _Pragma("linkage_name __ifftrad4_fr16")
        void ifftrad4_fr16 (const complex_fract16 *_in, complex_fract16 * _t, 
                        complex_fract16 *_out, const complex_fract16 _w[], 
                        int _wst, int _n, int _b, int _s);




/* * * *        cfft2d      * * * *
 *
 *    NxN point 2 dimensional complex input 
 * 
 */

                       _Pragma("linkage_name __cfft2d_fr16")
        void cfft2d_fr16 (const complex_fract16 *_in, complex_fract16 * _t, 
                        complex_fract16 *_out, const complex_fract16 _w[], 
                        int _wst, int _n, int _b, int _s );




/* * * *        rfft2d      * * * *
 *
 *    NxN point 2 dimensional real input FFT
 * 
 */

                       _Pragma("linkage_name __rfft2d_fr16")
        void rfft2d_fr16 (const fract16 *_in, complex_fract16 * _t, 
                        complex_fract16 *_out, const complex_fract16 _w[], 
                        int _wst, int _n, int _b, int _s );




/* * * *        ifft2d      * * * *
 *
 *    NxN point 2 dimensional inverse FFT
 * 
 */

                       _Pragma("linkage_name __ifft2d_fr16")
        void ifft2d_fr16 (const complex_fract16 *_in, complex_fract16 * _t, 
                        complex_fract16 *_out, const complex_fract16 _w[], 
                        int _wst, int _n, int _b, int _s );




/* * * *        a_compress      * * * *
 *
 *    A-law compression
 * 
 */

                       _Pragma("linkage_name __a_compress")
        void a_compress (const short _input[], short _output[], int _n);




/* * * *        a_expand      * * * *
 *
 *    A-law expansion
 * 
 */

                       _Pragma("linkage_name __a_expand")
        void a_expand (const short _input[], short _output[], int _n);




/* * * *        mu_compress      * * * *
 *
 *    mu compression
 * 
 */

                       _Pragma("linkage_name __mu_compress")
        void mu_compress (const short _input[], short _output[], int _n);




/* * * *        mu_expand      * * * *
 *
 *    mu expansion
 * 
 */

                       _Pragma("linkage_name __mu_expand")
        void mu_expand (const short _input[], short _output[], int _n);




#ifdef __cplusplus
 }      // end extern "C"
#endif 


#endif   /* __FILTER_DEFINED  (include guard) */


